//****************************************************************************** //
// $Id: $
// 
// Original Author: Gordon Roxburgh // Creation Date: 15th November 2016
// $Author: $
// $Date: $
// $Revision: $
//
//*****************************************************************************
//
// Description: Example code showing how to use the new key and tag feature with 
// Cobham wirelss analysis libraries. ( specifically WLAN 802.11AC )
//
// Copyright (c) 2005-20016, Aeroflex International Ltd.
//
//
//*****************************************************************************

#include "stdafx.h"
#include <stdio.h>
#include <vector>
#include <fstream>
#include <string>
#include <iostream>
#include <direct.h>
#include <math.h>
#include "afDigitizerDll.h"
#include "afSigGenDll.h"
#include <time.h>       /* clock_t, clock, CLOCKS_PER_SEC */
#include "afWlanDll.h"


typedef enum {VST_5644R, VST_5645R, VST_5646R} model_t;

typedef enum { LO_Above, LO_Below, LO_AUTO, LO_None } loPosition_t;

static const double MaximumFrequency = 6e9;

static const double MinimumFrequencyRegion1 = 65e6;
static const double MaximumFrequencyRegion1  = 109e6;
static const double MinimumFrequencyRegion2_5644R_5645R = 109e6;
static const double MaximumFrequencyRegion2_5644R_5645R = 6e9;
static const double MinimumFrequencyRegion2_5646R = 109e6;
static const double MaximumFrequencyRegion2_5646R = 200e6;
static const double MinimumFrequencyRegion3_5646R = 200e6;
static const double MaximumFrequencyRegion3_5646R = 6e9;
static const double MaximumBandwidthRegion1_5644R_5645R = 10e6;
static const double MaximumBandwidthRegion2_5644R_5645R = 27e6;

static const double MaximumBandwidthRegion1_5646R = 10e6;
static const double MaximumBandwidthRegion2_5646R = 27e6;
static const double MaximumBandwidthRegion3_5646R = 38e6;

//****************************************************************************** 
// Function          : DumpXY
// Globals Used      : None
// Globals Modified  : None
// Description       : Writes out the xData, yData into a csv file.
//******************************************************************************
void DumpXY(char* fileName, double* xData, double* yData, int NumberOfSamples)
{
	  std::ofstream myfile;	  

      myfile.open (fileName);     
      myfile << "X,Y\n";
      
	  for( int index = 0; index < NumberOfSamples; index++)
	  {		 
		  myfile << xData[index] << "," << yData[index] <<  "\n";
	  }
	  
      myfile.close();     

	  std::cout << "File Written : " << fileName << std::endl;
}


//****************************************************************************** 
// Function          : ErrorCheckVSA
// Globals Used      : None
// Globals Modified  : None
// Description       : Using a RFSA VISA session and a status code the status
// code is checked to see if it is either no error, an error or a warning.
//******************************************************************************
void ErrorCheckVSA(afDigitizerInstance_t Vsa, long ErrorCode)
{
	char errMsg[256];

	if ( ErrorCode > 0 ) // Warning
	{		
		// Retrieve the error message based on the status code.
		afDigitizerDll_ErrorMessage_Get(Vsa, errMsg,256);

		std::cout << "Warning : ";

		// Loop through the array of chars printing them out to the STDOUT.
		for( int i = 0 ; i < strlen(errMsg) ; i ++ )
		{
			std::cout << errMsg[i]; 
		}
		std::cout << std::endl;		
	}
	else if ( ErrorCode < 0 ) // Error
	{
		char errMsg[256];

		// Retrieve the error message based on the status code.
		afDigitizerDll_ErrorMessage_Get(Vsa, errMsg, 256);

		std::cout << "Error : ";			

		// Loop through the array of chars printing them out to the STDOUT.
		for( int i = 0 ; i < strlen(errMsg) ; i ++ )
		{
			std::cout << errMsg[i]; 
		}
		std::cout << std::endl;		
	}	
}

//****************************************************************************** 
// Function          : ErrorCheckVSG
// Globals Used      : None
// Globals Modified  : None
// Description       : Using a RFSG VISA session and a status code the status
// code is checked to see if it is either no error, an error or a warning.
//******************************************************************************
void ErrorCheckVSG(afSigGenInstance_t Vsg, long ErrorCode)
{
	char errMsg[256];

	if ( ErrorCode > 0 ) // Warning
	{		
		// Retrieve the error message based on the status code.
		afSigGenDll_ErrorMessage_Get(Vsg,errMsg,256);

		std::cout << "Warning : ";

		// Loop through the array of chars printing them out to the STDOUT.
		for( int i = 0 ; i < strlen(errMsg) ; i ++ )
		{
			std::cout << errMsg[i]; 
		}
		std::cout << std::endl;		
	}
	else if ( ErrorCode < 0 ) // Error
	{
		char errMsg[256];

		// Retrieve the error message based on the status code.
		afSigGenDll_ErrorMessage_Get(Vsg,errMsg,256);

		std::cout << "Error : ";			

		// Loop through the array of chars printing them out to the STDOUT.
		for( int i = 0 ; i < strlen(errMsg) ; i ++ )
		{
			std::cout << errMsg[i]; 
		}
		std::cout << std::endl;		
	}	
}

//****************************************************************************** 
// Function          : BootVSA
// Globals Used      : None
// Globals Modified  : None
// Description       : Boots the VSA.
//******************************************************************************
void BootVSA(afDigitizerInstance_t Mydigitizer,char* VsaLo,char* VsaRf)
{
	long errorCode;
	errorCode = afDigitizerDll_BootInstrument(Mydigitizer,VsaLo,VsaRf, AF_FALSE);
	ErrorCheckVSA(Mydigitizer,errorCode);

	// Set the LO reference to external.
	errorCode = afDigitizerDll_LO_Reference_Set(Mydigitizer, afDigitizerDll_lormExternalTerminated);
	ErrorCheckVSA(Mydigitizer,errorCode);

	errorCode = afDigitizerDll_RF_Reference_Set(Mydigitizer,afDigitizerDll_rfrmExternalDaisy);
	ErrorCheckVSA(Mydigitizer,errorCode);

	errorCode = afDigitizerDll_RF_ExternalReference_Set(Mydigitizer,afDigitizerDll_erLockTo10MHz);
	ErrorCheckVSA(Mydigitizer,errorCode);

	std::cout << "VSA Configured to lock to an external 10Mhz reference and terminate the 10Mhz reference on the paired 301(x)." << std::endl;			
}

//****************************************************************************** 
// Function          : BootVSG
// Globals Used      : None
// Globals Modified  : None
// Description       : Boots the VSG.
//******************************************************************************
void BootVSG(afSigGenInstance_t MySigGen, char* VsgLo,char* VsgRf)
{
	long errorCode;
	errorCode = afSigGenDll_BootInstrument(MySigGen,VsgLo,VsgRf,AF_FALSE);
	ErrorCheckVSG(MySigGen,errorCode);

	 // Set LO to use OCXO.
	errorCode = afSigGenDll_LO_Reference_Set( MySigGen, afSigGenDll_lormOCXO);
    ErrorCheckVSG(MySigGen,errorCode);

	// Now set VCO to be external.
    errorCode =  afSigGenDll_VCO_ExternalReference_Set(MySigGen,AF_TRUE);		
    ErrorCheckVSG(MySigGen,errorCode);


	std::cout << "VSG Configured to Generate 10Mhz Reference." << std::endl;			
}

//****************************************************************************** 
// Function          : CloseVsa
// Globals Used      : None
// Globals Modified  : None
// Description       : Closes the VSA object down.
//******************************************************************************
void CloseVsa(afDigitizerInstance_t Vsa)
{
	std::cout << "Destroying VSA Object." << std::endl;
	long status = afDigitizerDll_DestroyObject(Vsa);
		
	if (status != 0 )
	{
		std::cout << "An error occured shutting the VSG down." << std::endl;
	}
}

//****************************************************************************** 
// Function          : CloseVsg
// Globals Used      : None
// Globals Modified  : None
// Description       : Closes the VSG object down.
//******************************************************************************
void CloseVsg(afDigitizerInstance_t Vsg)
{
	std::cout << "Destroying VSG Object." << std::endl;
	long status = afSigGenDll_DestroyObject(Vsg);
		
	if (status != 0 )
	{
		std::cout << "An error occured shutting the VSG down." << std::endl;
	}
}

//****************************************************************************** 
// Function          : CloseWLAN
// Globals Used      : None
// Globals Modified  : None
// Description       : 
//******************************************************************************
void CloseWLAN(afWlanObjectID wlanObj)
{
	std::cout << "Destroying WLAN Analysis Object." << std::endl;
	long status = afWlanDll_DestroyObject(wlanObj);
	
	if (status != 0 )
	{
		std::cout << "An error occured shutting the WLAN Analysis Library Down" << std::endl;
	}
}

//****************************************************************************** 
// Function          : BootRFSG
// Globals Used      : None
// Globals Modified  : None
// Description       : Generates a CW tone at a given frequency and level.
//******************************************************************************
void PlayAiqWaveform(afSigGenInstance_t Vsg, char* fileNameAiq, double frequency, double level)
{
	long errorCode;
	// Set the MODE to manual and stop the ARB playing if it was playing.
	errorCode = afSigGenDll_Mode_Set(Vsg,afSigGenDll_mManual);
	ErrorCheckVSG(Vsg,errorCode);

	// The Arb file can only be stopped if it's currently playing and this can only be determined if the 
	// modulation source is set to ARB.
	afSigGenDll_msModulationSource_t currentManualModulation;

	errorCode = afSigGenDll_Manual_ModulationSource_Get(Vsg,&currentManualModulation);
	ErrorCheckVSG(Vsg,errorCode);

	if (currentManualModulation == afSigGenDll_msARB)
	{
		errorCode = afSigGenDll_Manual_ArbStopPlaying(Vsg);
		ErrorCheckVSG(Vsg,errorCode);
	}

	// Restore the Manual Mode settings for Frequency, Level and State. Also as we're requesting a CW tone then set
	// the Modulation source to CW.
	errorCode = afSigGenDll_Manual_ModulationSource_Set(Vsg, afSigGenDll_msARB);
	ErrorCheckVSG(Vsg,errorCode);

	// Set tge frequency, waveform to play, level and RF state.

	errorCode = afSigGenDll_Manual_ArbFile_Set(Vsg,fileNameAiq);
	ErrorCheckVSG(Vsg,errorCode);

	errorCode = afSigGenDll_Manual_Frequency_Set(Vsg, frequency);
	ErrorCheckVSG(Vsg,errorCode);

	errorCode = afSigGenDll_Manual_Level_Set(Vsg,level);
	ErrorCheckVSG(Vsg,errorCode);

	errorCode = afSigGenDll_Manual_RfState_Set(Vsg,AF_TRUE);
	ErrorCheckVSG(Vsg,errorCode);
}

//****************************************************************************** 
// Function          : BootRFSG
// Globals Used      : None
// Globals Modified  : None
// Description       : Generates a CW tone at a given frequency and level.
//******************************************************************************
void WaitForTrigger(afDigitizerInstance_t Vsa, int timeOutSeconds)
{
	long errorCode;
	// Wait for trigger (not necessary with immediate trigger)
	afDigitizerDll_tsTrigSource_t triggerSource;

	errorCode = afDigitizerDll_Trigger_Source_Get( Vsa, &triggerSource);
	ErrorCheckVSA(Vsa,errorCode);

	if (triggerSource == afDigitizerDll_tsSW_TRIG)
	{
		// Do nothing.                
	}
	else
	{
		AFBOOL detected = 0;
		
		clock_t startTime;
		clock_t stopTime;
		float elapsedTime;

		startTime = clock();

		do
		{
			errorCode = afDigitizerDll_Trigger_Detected_Get(Vsa,&detected);
			ErrorCheckVSA(Vsa,errorCode);			
			stopTime = clock();
			elapsedTime=(stopTime-startTime)/CLOCKS_PER_SEC;
		} while (detected == AF_FALSE && elapsedTime < timeOutSeconds);	

		if (detected == AF_FALSE)
		{
			errorCode = afDigitizerDll_Capture_IQ_Abort(Vsa);
			ErrorCheckVSA(Vsa,errorCode);
			std::cout << "Trigger Not Detected within timeout period" << std::endl;
		}
	}
}

//****************************************************************************** 
// Function          : Capture
// Globals Used      : None
// Globals Modified  : None
// Description       : Captures IQ for a specified dwell time at a specified frequency and input level at a specified sample rate.
//******************************************************************************
void Capture(afDigitizerInstance_t Vsa, double CarrierFrequency, double ReferenceLevel, unsigned long NumberOfSamples, double SampleRate, double* LevelCorrection, float* iBuffer, float* qBuffer, double* Key, double* Tag )
{
	long errorCode;

	//double levelCorrection = double.MinValue;
	int timeoutSeconds = 5;            
	AFBOOL captComplete = AF_FALSE;
	AFBOOL overloaded;

	// Following three commands take the digitizer from any state ( such as list mode ) and puts it into a basic instanious mode.
	// Configure the list mode address source to be manual.
	errorCode = afDigitizerDll_ListMode_AddressSource_Set(Vsa,afDigitizerDll_liasManual);
	ErrorCheckVSA(Vsa,errorCode);

	// Set the LO trigger mode to be none.
	errorCode = afDigitizerDll_LO_Trigger_Mode_Set(Vsa, afDigitizerDll_lotmNone);
	ErrorCheckVSA(Vsa,errorCode);

	// Set the source of the LO Address source to be none.
	errorCode = afDigitizerDll_LO_Trigger_AddressedSource_Set(Vsa,afDigitizerDll_tsaNONE);
	ErrorCheckVSA(Vsa,errorCode);

	// set the current channel to 0 to start
	errorCode = afDigitizerDll_RF_CurrentChannel_Set(Vsa,0);
	ErrorCheckVSA(Vsa,errorCode);

	// Use the generic modulation format.
	errorCode = afDigitizerDll_Modulation_Mode_Set(Vsa, afDigitizerDll_mmGeneric);
	ErrorCheckVSA(Vsa,errorCode);

	// Set the sample-rate that capture will be made at.
	errorCode = afDigitizerDll_Modulation_GenericSamplingFrequency_Set(Vsa,SampleRate);
	ErrorCheckVSA(Vsa,errorCode);

	// Set the centre frequency for the capture.
	errorCode = afDigitizerDll_RF_CentreFrequency_Set(Vsa,CarrierFrequency);
	ErrorCheckVSA(Vsa,errorCode);

	// Set the LO position to be above.
	errorCode = afDigitizerDll_RF_LOPosition_Set(Vsa,afDigitizerDll_lopAbove);
	ErrorCheckVSA(Vsa,errorCode);

	// Set the input level for the capture.
	errorCode = afDigitizerDll_RF_RFInputLevel_Set(Vsa,ReferenceLevel);
	ErrorCheckVSA(Vsa,errorCode);

	// Set the data to be IQ samples.
	errorCode = afDigitizerDll_Capture_SampleDataType_Set(Vsa,afDigitizerDll_sdtIQData);
	ErrorCheckVSA(Vsa,errorCode);

	// Set the capture to be made at 16 bit resolution.
	errorCode = afDigitizerDll_Capture_IQ_Resolution_Set(Vsa,afDigitizerDll_iqr16Bit);
	ErrorCheckVSA(Vsa,errorCode);          

	// Trigger Setup

	// HardCoded to Free Run for the example code
	// Set the trigger to be software trigger type to be armed.
	errorCode = afDigitizerDll_Trigger_Source_Set(Vsa,afDigitizerDll_tsSW_TRIG);
	ErrorCheckVSA(Vsa,errorCode); 

	// Set the trigger source to be software.
	errorCode = afDigitizerDll_Trigger_SwTriggerMode_Set(Vsa,afDigitizerDll_swtArmed);
	ErrorCheckVSA(Vsa,errorCode); 

	//switch (TriggerType)
	//{
	//    case TriggerTypeEnum.FreeRun:
	//        {
	//            // Set the trigger to be software trigger type to be armed.
	//            errorCode = vsaObj.Trigger.Source_Set(afDigitizerDll_tsTrigSource_t.afDigitizerDll_tsSW_TRIG);
	//            ErrorCheckDigitizer(errorCode);

	//            // Set the trigger source to be software.
	//            errorCode = vsaObj.Trigger.SwTriggerMode_Set(afDigitizerDll_swtSwTrigMode_t.afDigitizerDll_swtArmed);
	//            ErrorCheckDigitizer(errorCode);
	//            break;
	//        }
	//    case TriggerTypeEnum.Internal:
	//        {
	//            errorCode = vsaObj.Trigger.Source_Set(afDigitizerDll_tsTrigSource_t.afDigitizerDll_tsINT_TRIG);
	//            ErrorCheckDigitizer(errorCode);

	//            errorCode = vsaObj.Trigger.IntTriggerAbsThreshold_Set(TriggerLevel);
	//            ErrorCheckDigitizer(errorCode);

	//            uint delaySamples = (uint)GlobalConstsAndHelperMethods.GetIqIndexFromTimeframe(SampleRate, PreTriggerDelayS); // We want a -20uS delay.
	//            //trace.TraceVerbose(string.Format("Pre-Trigger Delay of {0} samples.", preTriggerSample));
	//            Console.WriteLine("Pre-Trigger Delay of {0} s or {1} samples", PreTriggerDelayS, delaySamples);
	//            errorCode = vsaObj.Trigger.PreEdgeTriggerSamples_Set((uint)delaySamples);
	//            ErrorCheckDigitizer(errorCode);


	//            errorCode = vsaObj.Trigger.EdgeGatePolarity_Set(afDigitizerDll_egpPolarity_t.afDigitizerDll_egpPositive);
	//            ErrorCheckDigitizer(errorCode);

	//            errorCode = vsaObj.Trigger.IntTriggerMode_Set(afDigitizerDll_itmIntTriggerMode_t.afDigitizerDll_itmAbsolute);
	//            ErrorCheckDigitizer(errorCode);

	//            errorCode = vsaObj.Trigger.IntTriggerAbsTimeConst_Set(0);
	//            ErrorCheckDigitizer(errorCode);

	//            errorCode = vsaObj.Trigger.TType_Set(afDigitizerDll_ttTrigType_t.afDigitizerDll_ttEdge);
	//            ErrorCheckDigitizer(errorCode);

	//            break;
	//        }
	//    case TriggerTypeEnum.External:
	//        {
	//            throw new NotSupportedException(string.Format("Trigger type {0} not supported.", TriggerType));
	//            //break;
	//        }
	//    default:
	//        {
	//            throw new NotSupportedException(string.Format("Trigger type {0} not supported.", TriggerType));
	//        }
	//}

	// Set and arm the trigger for the 'TotalDwellSamples' of samples.
	errorCode = afDigitizerDll_Capture_IQ_TriggerArm(Vsa,NumberOfSamples);
	ErrorCheckVSA(Vsa,errorCode);

	// Wait for trigger.
	WaitForTrigger(Vsa,timeoutSeconds);

	// Check for the capture to be complete before continuing.
	//Stopwatch swCaptComplete = Stopwatch.StartNew();
	clock_t startTime;
	clock_t stopTime;
	float elapsedTime;

	startTime = clock();

	// Wait for timeout to occur or the capture to be complete.
	do
	{
		errorCode = afDigitizerDll_Capture_IQ_CaptComplete_Get(Vsa,&captComplete);
		ErrorCheckVSA(Vsa,errorCode);
		stopTime = clock();
		elapsedTime=(stopTime-startTime)/CLOCKS_PER_SEC;
	} while (captComplete == AF_FALSE && elapsedTime < timeoutSeconds);


	// Now check to see if we exited the loop above due to a timeout of becuase we completed the capture.
	if (captComplete == AF_FALSE)
	{
		std::cout << "Capture not complete after 5 ms so capture aborted as something has gone wrong" << std::endl;
	}

	// Now fetch back the whole IQ in one chunk but with the tag and key ( which is needed for WLAN analysis )
	errorCode = afDigitizerDll_Capture_IQ_CaptMemWithKey(Vsa,NumberOfSamples, iBuffer, qBuffer, Tag, Key );
	ErrorCheckVSA(Vsa,errorCode);


	// Get the level correction.
	errorCode = afDigitizerDll_RF_LevelCorrection_Get( Vsa, LevelCorrection);
	ErrorCheckVSA(Vsa,errorCode);

	// Now we've captured update the ADCOverload property with the digitizers ADC status.                   
	errorCode = afDigitizerDll_Capture_IQ_ADCOverload_Get(Vsa, &overloaded);
	ErrorCheckVSA(Vsa,errorCode);

	if (overloaded == AF_TRUE)
	{
		std::cout << "VSA Capture Overloaded!" << std::endl;
	}


	// Always call abort after a capture.
	errorCode = afDigitizerDll_Capture_IQ_Abort(Vsa);
	ErrorCheckVSA(Vsa,errorCode);

}


//****************************************************************************** 
// Function          : ErrorCheckWLAN
// Globals Used      : None
// Globals Modified  : None
// Description       : Using a Cobham WLAN Analysis instance and a status code the status
// code is checked to see if it is either no error, an error or a warning occured.
//******************************************************************************
void ErrorCheckWLAN(afWlanObjectID WlanObj, long Status)
{	
	char errMsg[256];

	if ( Status > 0 ) // Warning
	{
		// Retrieve the error message based on the status code.
		afWlanDll_GetErrorMsg(WlanObj, Status, errMsg, sizeof(errMsg));

		std::cout << "Warning : ";

		// Loop through the array of chars printing them out to the STDOUT.
		for( int i = 0 ; i < strlen(errMsg) ; i ++ )
		{
			std::cout << errMsg[i]; 
		}
		std::cout << std::endl;
	}
	else if ( Status < 0 ) // Error
	{
		// Retrieve the error message based on the status code.
		afWlanDll_GetErrorMsg(WlanObj, Status, errMsg, sizeof(errMsg));

		std::cout << "Error : ";			

		// Loop through the array of chars printing them out to the STDOUT.
		for( int i = 0 ; i < strlen(errMsg) ; i ++ )
		{
			std::cout << errMsg[i]; 
		}
		std::cout << std::endl;		
	}			

//	delete errMsg;
}



//****************************************************************************** 
// Function          : WlanAnalysis
// Globals Used      : None
// Globals Modified  : None
// Description       : Creates a WLAN Analysis object and tries to perform the Power,
// OBW and Modulation accuracy measurments on a 802.11AC VHT 40MHz 
void WlanAnalysis(int NumSamples, double SampleRate, float LevelCorrection, float* iBuffer, float* qBuffer, double Key, double Tag )
{
	long status;
	
	//
	// Configure Analysis Library
	//	
	AFWLAN_MEASUREMENT measurements = afWlanLocateBurst | afWlanBurstPower | afWlanMeasOccupiedBandwidth | afWlanModAccuracy;

	afWlanObjectID wlanObj;
	status = afWlanDll_CreateObject(&wlanObj);
	ErrorCheckWLAN(wlanObj,status);

	// Set level correction to account for VST IQ W -> mW and VST impedance.
	status = afWlanDll_RfLevelCal_Set(wlanObj, LevelCorrection);
	ErrorCheckWLAN(wlanObj,status);

	// Set for 802.11AC VHT signal.
	status = afWlanDll_AnalysisMode_Set(wlanObj, afWlanAnalysisMode11acVHT);
	ErrorCheckWLAN(wlanObj,status);

	// Auto detect signal fields.
	status = afWlanDll_AutoDetectSignalFields_Set(wlanObj, AF_TRUE);
	ErrorCheckWLAN(wlanObj,status);

	// Define signal bandwidth to be 40MHz.
	status = afWlanDll_ChannelBandwidth_Set(wlanObj, afWlanChannelBandwidth40MHz);
	ErrorCheckWLAN(wlanObj,status);

	status = afWlanDll_AntennaNumber_Set(wlanObj, 0);
	ErrorCheckWLAN(wlanObj,status);

	status = afWlanDll_BurstProfile_Mode_Set(wlanObj, afWlanAveragePower);
	ErrorCheckWLAN(wlanObj,status);

	// Set amplitude tracking to off.
	status = afWlanDll_PilotTracking_Amplitude_Set(wlanObj, AF_FALSE);
	ErrorCheckWLAN(wlanObj,status);

	// Set phase tracking to on.
	status = afWlanDll_PilotTracking_Phase_Set(wlanObj, AF_TRUE);
	ErrorCheckWLAN(wlanObj,status);

	// Set timing tracking to off.
	status = afWlanDll_PilotTracking_Timing_Set(wlanObj, AF_FALSE);
	ErrorCheckWLAN(wlanObj,status);

	// Define the OFDM equalization setting to be over the preamble only.
	status = afWlanDll_Ofdm_EqualizationMode_Set(wlanObj, afWlanOfdmEqPreamble);
	ErrorCheckWLAN(wlanObj,status);

	// Define the analysis library to auto-detect the number of symbols
	status = afWlanDll_NumSymbolsToAnalyse_AutoDetect_Set(wlanObj, AF_TRUE);
	ErrorCheckWLAN(wlanObj,status);

	status = afWlanDll_ChannelOffset_Set(wlanObj, 0);
	ErrorCheckWLAN(wlanObj,status);

	status = afWlanDll_CaptureOffset_Set(wlanObj, 0);
	ErrorCheckWLAN(wlanObj,status);

	// Spectrum parameters
	status = afWlanDll_SpectralMask_Type_Set(wlanObj, afWlanSpectralMask11ac);
	ErrorCheckWLAN(wlanObj,status);

	status = afWlanDll_Spectrum_AnalysisMode_Set(wlanObj, afWlanSpectrumAnalysisNonGated);
	ErrorCheckWLAN(wlanObj,status);

	status = afWlanDll_Spectrum_FreqAxis_Centre_Set(wlanObj, 1e9);
	ErrorCheckWLAN(wlanObj,status);

	// If performing any spectrum measurements, the analysis library needs to know the bandwidth of the
	// VSA to determine whether stitching is required.
	double instBW = 90e6; // The maximum for 3035x in the WLAN frequency range.

	status = afWlanDll_DigitizerSpan_Set(wlanObj, instBW);
	ErrorCheckWLAN(wlanObj,status);
	// Query the minimum measurement span required for the desired measurements
	//
	// Note that all analysis library parameter that impact the measurement span must be set before querying, namely:
	// Analysis Mode, Channel BW, Channel Offset, Spectral Mask Type
	double minMeasSpan;
	status = afWlanDll_GetMinMeasurementSpan(wlanObj, measurements, &minMeasSpan);
	ErrorCheckWLAN(wlanObj,status);

	if(minMeasSpan <= instBW)
	{
		// Minimum measurement span is less than the VSA bandwidth - no stitching required
		status = afWlanDll_MeasurementSpan_Set(wlanObj, minMeasSpan);
		ErrorCheckWLAN(wlanObj,status);
	}
	else
	{
		// Minimum measurement span exceeds the VSA bandwidth, so stitching will be required
		std::cout << "Signal bandwith exceeds HW bandwidth. Stiching not supported in this example" << std::endl;
	}

	// Query the recomended sample rate for the desired measurements
	//
	// Note that all analysis library parameters that impact the sample rate required must be set before querying, namely:
	// Analysis Mode, Channel BW, Measurement Span, Digitiser Span.
	double analysisSampleRate;
	status = afWlanDll_GetRecommendedSamplingFreq(wlanObj, measurements, &analysisSampleRate); // query recommended sample rate
	ErrorCheckWLAN(wlanObj,status);

	status = afWlanDll_SamplingFreq_Set(wlanObj, analysisSampleRate); // set actual sample rate that will be used for capture
	ErrorCheckWLAN(wlanObj,status);

	// Finally, query the number of stitches required.
	//
	// Note that this step is very important for spectrum measurements and must be performed even if you already know the number
	// of stitches that will be required, because this function also performs some internal initialisation of the spectrum measurements.
	long stitches;
	status = afWlanDll_Spectrum_NumStitches_Get(wlanObj, &stitches);
	ErrorCheckWLAN(wlanObj,status);

	// Perform the analysis.
	status = afWlanDll_AnalyseIQ(wlanObj, measurements, iBuffer, qBuffer, NumSamples,0,NumSamples,Tag,Key);
	ErrorCheckWLAN(wlanObj,status);

	long pos = -1;
	status = afWlanDll_Burst_Position_Get(wlanObj, &pos);
	ErrorCheckWLAN(wlanObj,status);

	long len = -1;
	status = afWlanDll_Burst_Length_Get(wlanObj, &len);
	ErrorCheckWLAN(wlanObj,status);

	float power = -1;
	status = afWlanDll_BurstProfile_AveragePower_Get(wlanObj, &power);
	ErrorCheckWLAN(wlanObj,status);

	float evm;
	status = afWlanDll_Evm_Rms_Get(wlanObj, &evm);
	ErrorCheckWLAN(wlanObj,status);

	long mcs = -1;
	status = afWlanDll_MCS_Get(wlanObj, &mcs);
	ErrorCheckWLAN(wlanObj,status);

	double obw;
	status = afWlanDll_OccupiedBandwidth_Get(wlanObj, &obw);
	ErrorCheckWLAN(wlanObj,status);

	std::cout << "\nResults" << std::endl;
	std::cout << "-------\n" << std::endl;
	std::cout << "Burst Power\t: " << power << " dBm" << std::endl;
	std::cout << "EVM\t\t: " << evm << " %" << std::endl;
	std::cout << "MCS\t\t: " << mcs << std::endl;
	std::cout << "OBW\t\t: " << obw << " Hz" << std::endl;


	// Destroy the WLAN object.
	CloseWLAN(wlanObj);
}

//****************************************************************************** 
// Function          : _tmain
// Globals Used      : None
// Globals Modified  : None
// Description       : Console App entry point.
//******************************************************************************
int _tmain(int argc, _TCHAR* argv[])
{
	// Declare the VSG and VSA variables.
	afDigitizerInstance_t DigitizerId;
	afSigGenInstance_t SigGenId;

	// Create an instance of the object
	afDigitizerDll_CreateObject(&DigitizerId);
	afSigGenDll_CreateObject(&SigGenId);

	// Reference to the VISA Addresses of the four PXI Modules.
	char* VsgLo = "PXI8::15::INSTR";
	char* VsgRf = "PXI8::14::INSTR";
	char* VsaLo = "PXI8::11::INSTR";
	char* VsaRf = "PXI8::10::INSTR";

	double sampleRate		= 80e6; // 80 MHz. This is hard coded but should be derived from the analysis libraries.
	double dwellTime		= 1e-3; // 1 ms.
	double frequency		= 1e9;
	double level			= -20;	// -20 dBm
	double headRoom			= 16; // 16 dB of headroom.
	double referenceLevel	= level + headRoom;
	double levelCorrection;
	int numberOfSamples		= (int)(dwellTime * sampleRate);
	float *iData			= new float[numberOfSamples];
	float *qData			= new float[numberOfSamples];
	double key;
	double tag;
	int myChoice			= 0;
	
	std::cout << "PXI3000 Example 8 - Using Key and Tag Licensing the Analysis Libraries" << std::endl;
	std::cout << "----------------------------------------------------------------------" << std::endl;
	std::cout << std::endl;
	// Boot and configure the RFSG.
	std::cout << "Booting and configuring VSG." << std::endl;
	BootVSG(SigGenId, VsgLo, VsgRf);
	std::cout << "Booting and configuring VSA." << std::endl;
	BootVSA(DigitizerId, VsaLo, VsaRf);
	std::cout << std::endl;
	std::cout << "********** Main Menu **********" << std::endl;
	std::cout << "(1): Perform a looped back WLAN 802.11AC measurement." << std::endl;	
	std::cout << "(2): Exit" << std::endl;
	
	do
	{
		std::cin >> myChoice;

		switch(myChoice)
		{
		case 1:					
			PlayAiqWaveform(SigGenId,"WLAN_11AC_40MHz_MCS7.aiq",frequency,level);
			Capture(DigitizerId,frequency,referenceLevel,numberOfSamples,sampleRate,&levelCorrection,iData,qData,&key,&tag);
			WlanAnalysis(numberOfSamples,sampleRate, levelCorrection,iData,qData,key,tag);
			break;					
		default:
			std::cout << "ERROR! You have selected an invalid choice." << std::endl;
			break;
		}	
		std::cout << "********** Main Menu **********" << std::endl;
		std::cout << "(1): Perform a looped back WLAN 802.11AC measurement." << std::endl;	
		std::cout << "(2): Exit" << std::endl;
	}while(myChoice !=2);
	
	// Cleanup
	CloseVsg(SigGenId);
	CloseVsa(DigitizerId);
	
	return 0;
}

